import pyglet

import layer.load
import layer.parse
import layer.command
import layer.colors
import layer.audio
import layer.keybind

from layer.sprite import Sprite, OrderedGroup, batch

class DialogueActor(layer.parse.ParseObject):
    moods = {}
    box_color = (255, 255, 255)
    text_color = "black"
    box_opacity = 255

    _thawers_ = dict(box_color=layer.colors.value)

    def __getstate__(self):
        d = super(DialogueActor, self).__getstate__()
        moods = d.get("moods", {})
        for m, v in moods.items():
            moods[m] = v.filename
    
    def __setstate__(self, d):
        super(DialogueActor, self).__setstate__(d)
        for m, v in self.moods.items():
            self.moods[m] = layer.load.image(v)
            

class DialogueBlock(layer.parse.ParseObject):
    actor = 0
    mood = "normal"
    text = ""
    choices = {}

class DialogueDef(layer.parse.ParseObject):
    load_spec = "*.dialogue"
    actors = {}
    blocks = []

class Dialogue(object):
    def __init__(self, dfn, window, world):
        self.window = window
        self.world = world
        self.player = world.player
        self.group = OrderedGroup(100)
        self.avatargroup = OrderedGroup(0, parent=self.group)
        self.boxgroup = OrderedGroup(1, parent=self.group)
        self.textgroup = OrderedGroup(2, parent=self.group)
        self.choicesprites = []
        self.left_spr = Sprite(group=self.avatargroup, filename="white")
        self.left_spr.flip_x = True
        self.right_spr = Sprite(group=self.avatargroup, filename="white")
        self.dfn = dfn
        self.last = "right"
        self.left = None
        self.right = None
        self.box = Sprite(filename="speechbubble.png", group=self.boxgroup)
        self.box.image.anchor_x = self.box.width / 2
        self.box.image.anchor_y = self.box.height / 2
        self.box.y = self.window.height * 0.66
        self.box.x = self.window.width * 0.5
        self.box.scale = 0.8
        self.box.flip_y = False
        self.choices = []
        self.label = pyglet.text.HTMLLabel(
            multiline=True,
            batch=batch,
            group=self.textgroup,
            height=int(self.box.height * 0.8),
            width=int(self.box.width * 0.7),
            anchor_x="center",
            anchor_y="center",
            )
        self.label.y = int(self.box.y)
        self.label.content_valign = "center"
        self.label.color = (255, 255, 255, 255)
        layer.keybind.push("Dialogue")
        self.set_frame(0)

    def prep_choices(self):
        for i, (choice, command) in enumerate(reversed(self.choices)):
            bg = Sprite(filename="bar.png", group=self.boxgroup)
            bg.scale_y = 0.6
            bg.y = bg.height * i
            bg.x = (self.window.width - bg.width) / 2
            choice = "%d: %s" % (len(self.choices) - i, choice)
            l = pyglet.text.Label(choice,
                                  anchor_x="center",
                                  anchor_y="center",
                                  font_size=14,
                                  width=bg.width * 0.8,
                                  multiline=True,
                                  batch=bg.batch,
                                  group=self.textgroup)
            l.color = (255 ,255, 255, 255)
            l.x = bg.x + bg.width / 2
            l.y = bg.y + bg.height / 2
            self.choicesprites.append(l)
            self.choicesprites.append(bg)
        if len(self.choices) > 4:
            self.box.opacity = 0
            self.label.text = ""
        
    def set_frame(self, frame):
        self.frame = frame
        block = self.dfn.blocks[frame]
        speaking = self.dfn.actors[block.actor]
        self.label.text = "<center><font color=%s>%s</font></center>" % (
            speaking.text_color, block.text)
        self.label.font_size = int(20 * self.box.scale)
        self.box.color = speaking.box_color
        self.box.opacity = speaking.box_opacity
        self.choices = sorted(block.choices.items())
        while self.choicesprites:
            self.choicesprites.pop().delete()

        if speaking == self.right:
            spr = self.right_spr
        elif speaking == self.left:
            spr = self.left_spr
        else:
            if self.last == "left":
                self.right = speaking
                spr = self.right_spr
                self.last = "right"
            else:
                self.left = speaking
                spr = self.left_spr
                self.last = "left"
        spr.image.opacity = 255
        try: spr.image = speaking.moods[block.mood]
        except KeyError:
            if "normal" in speaking.moods:
                spr.image = speaking.moods["normal"]
            else:
                spr.image.opacity = 0
        if self.left_spr == spr:
            self.left_spr.color = (255, 255, 255)
            self.right_spr.color = (200, 200, 200)
            self.box.flip_x = False
            self.box.x = self.window.width * 0.55
            self.label.x = int(self.box.x)
        elif self.right_spr == spr:
            self.right_spr.color = (255, 255, 255)
            self.left_spr.color = (200, 200, 200)
            self.box.flip_x = True
            self.box.x = self.window.width * 0.45
            self.label.x = int(self.box.x)
        max_width = self.window.width * 0.4
        max_height = self.window.height * 0.8
        self.left_spr.scale = min(1,
                                  max_width / self.left_spr.image.width,
                                  max_height / self.left_spr.image.height)
        self.right_spr.scale = min(1,
                                   max_width / self.right_spr.image.width,
                                   max_height / self.right_spr.image.height)
        self.left_spr.x = 0
        self.left_spr.y = 0

        self.right_spr.x = self.window.width - self.right_spr.width
        self.right_spr.y = 0

    def advance(self, choice=10000000000000):
        layer.audio.AudioEvent.Find("menu_beep").play()
        if self.choices:
            if self.choicesprites:
                try: exec self.choices[int(choice)][1]
                except IndexError: return True
                else: return self.frame < (len(self.dfn.blocks) - 1)
            else:
                self.prep_choices()
                return True
        else:
            try: self.set_frame(self.frame + 1)
            except IndexError: return False
            else: return True

    def destroy(self):
        self.box.delete()
        self.label.delete()
        self.left_spr.delete()
        self.right_spr.delete()
        layer.keybind.pop("Dialogue")
        while self.choicesprites:
            self.choicesprites.pop().delete()
